<?php
// require_once(__DIR__ . '../vendor/autoload.php');
require_once dirname(dirname(__FILE__)) . '/vendor/autoload.php';
require_once dirname(dirname(__FILE__)) . '/lib/Helpers.php';

require_once dirname(dirname(__FILE__)) . '/publitio_api.php';

// Please update xxxx with your key and yyyy with your secret

/** Load all of the necessary class files for the plugin */
use lsolesen\pel\PelDataWindow;
use lsolesen\pel\PelJpeg;
use lsolesen\pel\PelTiff;
use lsolesen\pel\Pel;
use lsolesen\pel\PelExif;
use lsolesen\pel\PelIfd;
use lsolesen\pel\PelTag;
use lsolesen\pel\PelEntryAscii;
use lsolesen\pel\PelEntryUserComment;
use lsolesen\pel\PelEntryByte;
use lsolesen\pel\PelEntryRational;

// use \Cloudinary\Uploader;


class Wp_Image_compression
{

    private $id;
    private $compression_settings = array();
    private $thumbs_data = array();
    private $backup_before_compression = 0;

    function __construct()
    {

        if (!class_exists('Optimisationio_Dashboard')) {
            require_once 'class-optimisationio-dashboard.php';
        }

        Optimisationio_Dashboard::init();

        require_once 'class-wp-image.php';

        $this->compression_settings = unserialize(get_option('_wpimage_options'));

        $this->backup_before_compression = isset($this->compression_settings['backup_before_compression']) ? $this->compression_settings['backup_before_compression'] : false;

        add_action('admin_enqueue_scripts', array(&$this, 'admin_scripts'));
        add_action('wp_ajax_wpimage_request', array(&$this, 'wpimage_media_library_ajax_callback'));
        add_action('manage_media_custom_column', array(&$this, 'fill_media_columns'), 10, 2);
        add_filter('manage_media_columns', array(&$this, 'add_media_columns'));
        //add_filter('wp_generate_attachment_metadata', array(&$this, 'optimize_thumbnails'));
        add_action('add_attachment', array(&$this, 'wpimage_media_uploader_callback'));

        add_action('wp_loaded', array($this, 'exe_before_wpheader'));

        if (!is_admin()) {
            //lazyl
            global $wpImagelazySizesDefaults;

            $settings = get_option('_wpimage_lazyload_options', $wpImagelazySizesDefaults);

            if (isset($settings) && $settings != '') {
                $settings = unserialize($settings);
            }

            if (isset($settings['enable_lazyload']) && $settings['enable_lazyload'] == 'true') {
                add_action('wp_enqueue_scripts', array(&$this, 'add_lazyloading_styles'), 1);
                add_action('wp_enqueue_scripts', array(&$this, 'add_lazyloading_scripts'), 200);
               // Run this later, so other content filters have run, including image_add_wh on WP.com
                add_filter('the_content', array(&$this, 'filter_images'), 200);
                add_filter('post_thumbnail_html', array(&$this, 'filter_images'), 200);
                add_filter('widget_text', array(&$this, 'filter_images'), 200);
                if ($settings['lazyload_iframe'] != 'false') {
                    add_filter('oembed_result', array(&$this, 'filter_iframes'), 200);
                    add_filter('embed_oembed_html', array(&$this, 'filter_iframes'), 200);
                }
                add_filter('get_avatar', array(&$this, 'filter_avatar'), 200);
            }
        }
    }

    private function get_js_config()
    {
        global $wpImagelazySizesDefaults;
        $settings = unserialize(get_option('_wpimage_lazyload_options', $wpImagelazySizesDefaults));
        return array(
            'expand' => $settings['lazyload_expand'],
            'preloadAfterLoad' => $settings['lazyload_preloadAfterLoad']
        );
    }

    private function _add_class($htmlString = '', $newClass)
    {

        $pattern = '/class="([^"]*)"/';

        // Class attribute set.
        if (preg_match($pattern, $htmlString, $matches)) {
            $definedClasses = explode(' ', $matches[1]);
            if (!in_array($newClass, $definedClasses, true)) {
                $definedClasses[] = $newClass;
                $htmlString = str_replace(
                    $matches[0],
                    sprintf('class="%s"', implode(' ', $definedClasses)),
                    $htmlString
                );
            }
        // Class attribute not set.
        } else {
            $htmlString = preg_replace('/(\<.+\s)/', sprintf('$1class="%s" ', $newClass), $htmlString);
        }

        return $htmlString;
    }

    private function validate_cloudinary_credentials($cloud_name = '', $api_key = '', $api_secret = '')
    {
        include_once 'lib/Api.php';
        include_once 'lib/Cloudinary.php';
        $ret = false;
        if ('' === $cloud_name || '' === $api_key || '' === $api_secret) {
            return $ret;
        }
        try {
            \Cloudinary::config(
                array(
                    "cloud_name" => $cloud_name,
                    "api_key" => $api_key,
                    "api_secret" => $api_secret
                )
            );
            $api = new \Cloudinary\Api();
            $ping = $api->ping();
            $ret = $ping && isset($ping['status']) && 'ok' === $ping['status'];
        } catch (Exception $e) {
        }
        return $ret;
    }

    /*
     *  Adds wpimage fields and settings to Settings->Media settings page
     */
    public function add_lazyloading_styles()
    {
        wp_enqueue_style('wpimage_lazyloading', plugins_url(ltrim('css/lazysizes.min.css', '/'), dirname(__FILE__)), array(), "1.1");
    }

    public function add_lazyloading_scripts()
    {
        global $wpImagelazySizesDefaults;
        $settings = unserialize(get_option('_wpimage_lazyload_options', $wpImagelazySizesDefaults));
        wp_enqueue_script('wpimage_lazyloading', plugins_url(ltrim('build/wp-lazysizes.min.js'), dirname(__FILE__)), array(), "1.1", false);

        if ($settings['lazyload_optimumx'] !== 'false') {
            wp_enqueue_script(
                'lazysizesoptimumx',
                plugins_url(ltrim('js/lazysizes/plugins/optimumx/ls.optimumx.min.js', '/'), dirname(__FILE__)),
                array(),
                "1.1",
                false
            );
        }

        wp_localize_script('wpimage_lazyloading', 'lazySizesConfig', $this->get_js_config());
    }

    public function filter_images($content, $type = 'ratio')
    {
        global $wpImagelazySizesDefaults;
        $settings = unserialize(get_option('_wpimage_lazyload_options', $wpImagelazySizesDefaults));
        if (is_feed()
            || intval(get_query_var('print')) == 1
            || intval(get_query_var('printpage')) == 1
            || strpos($_SERVER['HTTP_USER_AGENT'], 'Opera Mini') !== false) {
            return $content;
        }

        $respReplace = 'data-sizes="auto" data-srcset=';

        if ($settings['lazyload_optimumx'] != 'false') {
            $respReplace = 'data-optimumx="' . $settings['lazyload_optimumx'] . '" ' . $respReplace;
        }

        $matches = array();
        $skip_images_regex = '/class=".*lazyload.*"/';
        $placeholder_image = apply_filters(
            'lazysizes_placeholder_image',
            'data:image/gif;base64,R0lGODlhAQABAAAAACH5BAEKAAEALAAAAAABAAEAAAICTAEAOw=='
        );
        preg_match_all('/<img\s+.*?>/', $content, $matches);

        $search = array();
        $replace = array();

        foreach ($matches[0] as $imgHTML) {

            // Don't to the replacement if a skip class is provided and the image has the class.
            if (!(preg_match($skip_images_regex, $imgHTML))) {

                $replaceHTML = preg_replace('/<img(.*?)src=/is', '<img$1src="' . $placeholder_image . '" data-src=', $imgHTML);
                $replaceHTML = preg_replace('/<img(.*?)srcset=/is', '<img$1srcset="" data-srcset=', $replaceHTML);


                $replaceHTML = $this->_add_class($replaceHTML, 'lazyload');

                $replaceHTML .= '<noscript>' . $imgHTML . '</noscript>';

                if ($type == 'ratio' && $settings['lazyload_intrinsicRatio'] != 'false') {
                    if (preg_match('/width=["|\']*(\d+)["|\']*/', $imgHTML, $width) == 1
                        && preg_match('/height=["|\']*(\d+)["|\']*/', $imgHTML, $height) == 1) {

                        $ratioBox = '<span class="intrinsic-ratio-box';
                        if (preg_match('/(align(none|left|right|center))/', $imgHTML, $align_class) == 1) {
                            $ratioBox .= ' ' . $align_class[0];
                            $replaceHTML = str_replace($align_class[0], '', $replaceHTML);
                        }
                        if ($settings['lazyload_intrinsicRatio'] == 'animated') {
                            $ratioBox .= ' lazyload" data-expand="-1';
                        }

                        $ratioBox .= '" style="max-width: ' . $width[1] . 'px; max-height: ' . $height[1] . 'px;';

                        $ratioBox .= '"><span class="intrinsic-ratio-helper" style="padding-bottom: ';
                        $replaceHTML = $ratioBox . (($height[1] / $width[1]) * 100) . '%;"></span>'
                            . $replaceHTML . '</span>';
                    }
                }

                array_push($search, $imgHTML);
                array_push($replace, $replaceHTML);
            }
        }
        $search = array_unique($search);
        $replace = array_unique($replace);
        $content = str_replace($search, $replace, $content);

        return $content;
    }

    public function filter_avatar($content)
    {
        return $this->filter_images($content, 'noratio');
    }

    public function filter_iframes($html)
    {
        return false === strpos($html, 'iframe') ? $html : $this->_add_class($html, 'lazyload');
    }

    public function admin_scripts($hook)
    {

        $plugin_dir = dirname(__FILE__);

        if ('options-media.php' === $hook || 'upload.php' === $hook) {

            wp_enqueue_style('wpimage_admin_style', plugins_url('css/admin.css', $plugin_dir));
            wp_enqueue_style('tipsy-style', plugins_url('css/tipsy.css', $plugin_dir));
            wp_enqueue_style('modal-style', plugins_url('css/jquery.modal.css', $plugin_dir));

            wp_enqueue_style('typeahead', plugins_url('css/jquery.typeahead.min.css', $plugin_dir));

            wp_enqueue_script('jquery');
            wp_enqueue_script('tipsy-js', plugins_url('js/jquery.tipsy.js', $plugin_dir), array('jquery'));
            wp_enqueue_script('async-js', plugins_url('js/async.js', $plugin_dir));
            wp_enqueue_script('modal-js', plugins_url('js/jquery.modal.min.js', $plugin_dir), array('jquery'));
            wp_enqueue_script('ajax-script', plugins_url('js/ajax.js', $plugin_dir), array('jquery'));

            wp_enqueue_script('typeahead', plugins_url('js/jquery.typeahead.min.js', $plugin_dir), array('jquery'));

            wp_localize_script('ajax-script', 'ajax_object', array('ajax_url' => admin_url('admin-ajax.php')));
        }

        if ('toplevel_page_optimisationio-dashboard' === $hook) {
            wp_enqueue_script('stripe-js', 'https://checkout.stripe.com/checkout.js');
            wp_register_script('stripe-custom-js', plugins_url('js/stripe.js', $plugin_dir), 20, "1.3", true);
            wp_enqueue_script('stripe-custom-js');
        }
    }

    public function get_api_status()
    {
        return true;
    }

    /**
     *  Handles optimizing already-uploaded images in the  Media Library
     */
    public function wpimage_media_library_ajax_callback()
    {

        $post_req = $_POST;     // Input var okay.

        $image_id = (int)$post_req['id'];
        $type = isset($post_req['type']) ? $post_req['type'] : false;
        $already_optimised = isset($post_req['optimised']) ? (int)$post_req['optimised'] : 0;
        $this->id = $image_id;

    // Publitio Image Conversion start...................................................................................... 

        //get the setting from compress images tab in optimisation page
        $image_quality = get_option( 'wpimages_quality_auto' );
        $bmp_to_jpg = get_option( 'wpimages_bmp_to_jpg' );
        $png_to_jpg = get_option( 'wpimages_png_to_jpg' );
        // debug($image_quality); debug($bmp_to_jpg); debug($png_to_jpg); die();

            $image_path = get_attached_file($image_id);
            $filename = basename($image_path);
            $this_img_extension = "";
            $extensions = array('.jpg', '.JPG', '.png' ,'.PNG' ,'.jpeg' ,'.JPEG');
                foreach ($extensions as $key => $extension) {
                    if (strpos($filename, $extension) !== false) {
                        $this_img_extension = $extension;
                        $img_name = str_replace($extension,"",$filename);
                    }
                }

            $publitio_api = new PublitioAPI('hxr7aqQDXG6WyLMApSjc', 'SmSt4vSBRtBW2m0kLAx5HsPikzhNwLuj');
            //first we will check if this image is already uploaded or not
                $reload = 0;
                // list all files to check for this recently uploaded file by file name
                $get_all_files = $publitio_api->call("/files/list", "GET", array('offset' => '0', 'limit' => '10'));
                $all_files = json_decode($get_all_files);
                // debug($all_files->files); die();

                //first of all find any files present in our publitio db, that were not deleted and delete them
                $allFiles = $all_files->files;
                foreach ($allFiles as $key => $file) {
                    $publito_img_id = $file->id;
                    $publitio_api->call("/files/delete/".$publito_img_id, "DELETE"); 
                }

                //and now,first upload the original file from our media library to be called later in other format
                $response = $publitio_api->upload_file($image_path, "file", array('name' => $filename, 'position' => 'top-right', 'padding' => '20')); 
                    $uploaded_file = json_decode($response);
                        if ($uploaded_file->success == 1) {
                            $publito_img_id = $uploaded_file->id;
                            $publito_img_extension = $file->extension;
                            $publito_img_src = $file->url_preview;
                            $publito_img_download_src = $file->url_download;

                            //now get the desired format image from url of this original format uploaded image
                             if ($png_to_jpg == '1' && $publito_img_extension == 'png') {
                                 $new_format = ".jpg";
                             }
                             if ($bmp_to_jpg == '1' && $publito_img_extension == 'bmp') {
                                 $new_format = ".jpg";
                             }
                            if ($new_format !== null) {
                                $new_image_src = "https://media.publit.io/file/".$publito_img_id."".$new_format;
                                $new_image = '<img src="'.$new_image_src.'" width="200px">';
                                $new_image_name = $img_name."".$new_format;
                                $old_image_id = $image_id;
                            
                                //upload this image to media library with new image format directly from source
                                $this->wpimage_upload_from_url($old_image_id, $new_image_name, $new_image_src);

                                //at last delete this uploaded file with publitio image id 
                                $response = $publitio_api->call("/files/delete/".$publito_img_id, "DELETE"); 
                                $reload = 1;
                                wp_delete_attachment( $image_id ); //at last delete this uploaded file from media library
                            }
                        }
                
                if ($reload == 1) {
                    echo json_encode(array('reload' => 'reload'));
                    die();
                }
// Publitio Image Conversion ends......................................................................................  

//in case when internet is down, above publitio code will not work, then stop image from being optimised below according to settings, before it converts to derired formats.         
$permission = 0;       
if ($png_to_jpg == '1' && $this_img_extension == '.png') {
    $permission = 1;
}
if ($bmp_to_jpg == '1' && $this_img_extension == '.bmp') {
    $permission = 1;
}
//permission = 1 means, first convert to jpg above from publitio, then run below code in new jpg format image

        if (wp_attachment_is_image($image_id) && $permission == 0) {

            $image_path = false;
            $original_image_path = get_attached_file($image_id);

            if ($already_optimised) {
                $wp_meta = wp_get_attachment_metadata($image_id);
                if (isset($wp_meta['file'])) {
                    $uploads = wp_upload_dir();
                    $image_path = $uploads['basedir'] . '/optimisationio_media_backup/' . basename($wp_meta['file']);
                }
            }

            $image_path = $image_path ? $image_path : $original_image_path;

            $status = $this->get_api_status();

            if (false === $status) {
                $kv['error'] = 'There is a problem with your credentials. Please check them in the wp-image.co.uk settings section of Media Settings, and try again.';
                update_post_meta($image_id, '_wpimage_size', $kv);
                echo json_encode(array('error' => $kv['error']));
                exit;
            }

            $result = $this->optimize_image($image_path, $already_optimised ? false : $this->backup_before_compression);

            if ($result && $result['url']) {

                $savings_percentage = (int)$result['saved_bytes'] / (int)$result['original_size'] * 100;

                $kv = array(
                    'success' => true,
                    'meta' => wp_get_attachment_metadata($image_id),
                    'original_size' => self::pretty_kb($result['original_size']),
                    'compressed_size' => self::pretty_kb($result['bytes']),
                    'saved_bytes' => self::pretty_kb($result['saved_bytes']),
                    'savings_percent' => round($savings_percentage, 2) . '%',
                    'backup_before_compression' => $result['backup_before_compression']
                );

                if ($this->replace_image($original_image_path, $result['url'])) {

                    // Get metadata for thumbnails.
                    /*$image_data = wp_get_attachment_metadata( $image_id );
                    $this->optimize_thumbnails( $image_data );*/

                    // Store compressed info to DB.
                    update_post_meta($image_id, '_wpimage_size', $kv);

                    // Compress thumbnails, store that data too.
                    /*$compressed_thumbs_data = get_post_meta($image_id, '_compressed_thumbs', true);
                    if (!empty($compressed_thumbs_data)) {
                        $kv['thumbs_data'] = $compressed_thumbs_data;
                    }*/
                } else {
                    $kv = array('error' => 'Could not overwrite original file. Please ensure that your files are writable by plugins.');
                }

                $this->delete_resources($result['public_id']);
                print_r(json_encode($kv));
            } else {

                // error or no optimization
                if (file_exists($image_path)) {

                    $kv = array(
                        'original_size' => self::pretty_kb(filesize($image_path)),
                        'error' => $result['error'],
                        // 'type' => $result['type'],
                    );

                    if ('This image can not be optimized any further' === $kv['error']) {
                        $kv['compressed_size'] = 'No savings found';
                        $kv['no_savings'] = true;
                    }

                    update_post_meta($image_id, '_wpimage_size', $kv);
                } else {
                    // file not found
                }

                print_r(json_encode($result));
            }

        }

        die();
    }

    public function wpimage_upload_from_url($old_image_id, $new_image_name, $new_image_src) {

        $get_image_src = $new_image_src;

        if ( !function_exists('media_handle_upload') ) {
            require_once(ABSPATH . "wp-admin" . '/includes/image.php');
            require_once(ABSPATH . "wp-admin" . '/includes/file.php');
            require_once(ABSPATH . "wp-admin" . '/includes/media.php');
        }

        $url = $get_image_src;
        $tmp = download_url( $url );
        if( is_wp_error( $tmp ) ){
            // download failed, handle error
        }
        $post_id = '0';  //just for relationship, can be set to 0 also
        $desc = $new_image_name;
        $file_array = array();

        preg_match('/[^\?]+\.(jpg|jpe|jpeg|gif|png|webp)/i', $url, $matches);
        $file_array['name'] = basename($matches[0]);
        $file_array['tmp_name'] = $tmp;
        // If error storing temporarily, unlink
        if ( is_wp_error( $tmp ) ) {
            @unlink($file_array['tmp_name']);
            $file_array['tmp_name'] = '';
        }
        // do the validation and storage stuff
        $new_imageId = media_handle_sideload( $file_array, $post_id, $desc );
        // $new_file_data = get_post_meta( $new_imageId, '_wp_attached_file' );
        // $new_file_attachment_metadata = get_post_meta( $new_imageId, '_wp_attachment_metadata' );

        // If error storing permanently, unlink
        if ( is_wp_error($new_imageId) ) {
            @unlink($file_array['tmp_name']);
            return $new_imageId;
        }
                                 
}

    /**
     *  Handles optimizing images uploaded through any of the media uploaders.
     */
    public function wpimage_media_uploader_callback($image_id)
    {

        $this->id = $image_id;

        if (wp_attachment_is_image($image_id)) {

            $settings = $this->compression_settings;
            $backup_before_compression = $settings['backup_before_compression'];
            $image_path = get_attached_file($image_id);
            $result = $this->optimize_image($image_path, $backup_before_compression);

            if ($result && isset($result['url'])) {

                $compressed_url = $result['url'];
                $savings_percentage = (int)$result['saved_bytes'] / (int)$result['original_size'] * 100;
                $kv['original_size'] = self::pretty_kb($result['original_size']);
                $kv['compressed_size'] = self::pretty_kb($result['bytes']);
                $kv['saved_bytes'] = self::pretty_kb($result['saved_bytes']);
                $kv['savings_percent'] = round($savings_percentage, 2) . '%';
                $kv['backup_before_compression'] = $settings['backup_before_compression'];
                $kv['success'] = true;
                $kv['meta'] = wp_get_attachment_metadata($image_id);

                if ($this->replace_image($image_path, $compressed_url)) {
                    update_post_meta($image_id, '_wpimage_size', $kv);
                    $this->delete_resources($result['public_id']);
                } else {
                    // writing image failed
                    $this->delete_resources($result['public_id']);
                }
            } else {

                // error or no optimization
                if (file_exists($image_path)) {

                    $kv['original_size'] = self::pretty_kb(filesize($image_path));
                    $kv['error'] = $result['error'];
                    $kv['type'] = $result['type'];

                    if ($kv['error'] == 'This image can not be optimized any further') {
                        $kv['compressed_size'] = 'No savings found';
                        $kv['no_savings'] = true;
                    }

                    update_post_meta($image_id, '_wpimage_size', $kv);
                } else {
                    // file not found
                }
            }
        }
    }

    public function show_credentials_validity()
    {

        $settings = $this->compression_settings;
        $status = $this->get_api_status();
        $url = admin_url() . 'images/';

        if ($status !== false && isset($status['active'])) {
            $url .= 'yes.png';
            echo '<p class = "apiStatus">Your credentials are valid <span class = "apiValid" style = "background:url(' . "'$url') no-repeat 0 0" . '"></span></p>';
        } else {
            $url .= 'no.png';
            echo '<p class = "apiStatus">There is a problem with your credentials <span class = "apiInvalid" style = "background:url(' . "'$url') no-repeat 0 0" . '"></span></p>';
        }
    }

    public function validate_options($input)
    {

        $valid = array();
        $error = '';
        $valid['api_lossy'] = $input['api_lossy'];

        if (!function_exists('curl_exec')) {
            $error = 'cURL not available. Wp image compression requires cURL in order to communicate with wp-image.co.uk servers. <br /> Please ask your system administrator or host to install PHP cURL, or contact support@wp-image.co.uk for advice';
        } else {
            $status = $this->get_api_status();

            if ($status !== false) {

                if (isset($status['active'])) {
                    if ($status['plan_name'] === 'Developers') {
                        $error = 'Developer API credentials cannot be used with this plugin.';
                    } else {
                        $valid['api_key'] = $input['api_key'];
                        $valid['api_secret'] = $input['api_secret'];
                    }
                } else {
                    $error = 'There is a problem with your credentials. Please check them from your wp-image.co.uk account.';
                }
            } else {
                $error = 'Please enter a valid wp-image.co.uk API key and secret';
            }
        }

        if (!empty($error)) {
            add_settings_error(
                'media',
                'api_key_error',
                $error,
                'error'
            );
        }
        return $valid;
    }

    public function show_lossy()
    {

        $options = get_option('_wpimage_options');
        $value = isset($options['api_lossy']) ? $options['api_lossy'] : 'lossy';

        $html = '<input type="radio" id="wpicompressor_lossy" name="_wpimage_options[api_lossy]" value="lossy"' . checked('lossy', $value, false) . '/>';
        $html .= '<label for="wpicompressor_lossy">Lossy</label>';

        $html .= '<input style="margin-left:10px;" type="radio" id="wpimage_lossless" name="_wpimage_options[api_lossy]" value="lossless"' . checked('lossless', $value, false) . '/>';
        $html .= '<label for="wpimage_lossless">Lossless</label>';

        echo $html;
    }

    public function add_media_columns($columns)
    {
        // $columns['quality'] = 'Quality';

        $columns['original_size'] = 'Original Size';
        $columns['geo'] = 'Geo';
        $columns['compressed_size'] = 'Compressed Size';
        return $columns;
    }

    public function fill_media_columns($column_name, $id)
    {

        $original_size = @filesize(get_attached_file($id));
        $original_size = self::pretty_kb($original_size);

        $options = get_option('_wpimage_options');
        $type = isset($options['api_lossy']) ? $options['api_lossy'] : 'lossy';

        if (0 === strcmp($column_name, 'original_size')) {
            if (wp_attachment_is_image($id)) {
                $meta = get_post_meta($id, '_wpimage_size', true);
                echo isset($meta['original_size']) ? $meta['original_size'] : $original_size;
            } else {
                echo $original_size;
            }
        } else if (0 === strcmp($column_name, 'compressed_size')) {

            if (wp_attachment_is_image($id)) {

                $meta = get_post_meta($id, '_wpimage_size', true);

                $backup_file_exists = false;
                if (isset($meta['meta'])) {

                    $uploads = wp_upload_dir();
                    $image_in_backup = $uploads['basedir'] . '/optimisationio_media_backup/';

                    if (isset($meta['meta']['file'])) {
                        $image_in_backup .= basename($meta['meta']['file']);
                        $backup_file_exists = file_exists($image_in_backup);
                    } else {

                        $wp_meta = wp_get_attachment_metadata($id);

                        if (isset($wp_meta['file'])) {
                            $image_in_backup .= basename($wp_meta['file']);
                            $backup_file_exists = file_exists($image_in_backup);
                        }
                    }
                }

                $image_url = wp_get_attachment_url($id);
                $filename = basename($image_url);

                if (isset($meta['compressed_size'])) {

                    $compressed_size = $meta['compressed_size'];

                    if ($meta['savings_percent'] >= 0) {
                        $savings_percentage = $meta['savings_percent'];
                        echo '<strong>' . $compressed_size . '</strong><br/><small>Savings:&nbsp;' . $savings_percentage . '</small>';

                        $thumbs_data = get_post_meta($id, '_compressed_thumbs', true);
                        $thumbs_count = count($thumbs_data);

                        if (!empty($thumbs_data)) {
                            echo '<small>' . $thumbs_count . ' thumbs optimized' . '</small>';
                        }
                    } else {
                        echo '<br/><small>No further optimization required</small>';
                    }

                    if (!empty($meta['no_savings'])) {
                        echo '<div class="noSavings"><strong>No savings found</strong><br /></div>';
                    } else if (isset($meta['error'])) {
                        $error = $meta['error'];
                        echo '<div class="wpimageErrorWrap"><a class="wpimageError" title="' . $error . '">Failed! Hover here</a></div>';
                    }
                }

                if (!isset($meta['compressed_size']) || $backup_file_exists) {
                    echo '<div class="buttonWrap">';
                    echo $backup_file_exists ? '<br/>' : '';
                    echo '<button data-setting="' . $type . '" type="button" class="wpimage_req" data-id="' . $id . '" id="wpimageid-' . $id . '" data-filename="' . $filename . '" data-url="' . $image_url . '" data-optimised="' . ($backup_file_exists ? 1 : 0) . '">';
                    echo $backup_file_exists ? 'Re-Optimize This Image' : 'Optimize This Image';
                    echo '</button>';
                    echo '<span class="wpimageSpinner"></span>';
                    echo '</div>';
                }

            } else {
                echo 'n/a';
            }
        } else if (0 === strcmp($column_name, 'geo')) {
            if (wp_attachment_is_image($id)) {

                $meta = get_post_meta($id, '_wpimage_size', true);

                $backup_file_exists = false;
                if (isset($meta['meta'])) {

                    $uploads = wp_upload_dir();
                    $image_in_backup = $uploads['basedir'] . '/optimisationio_media_backup/';

                    if (isset($meta['meta']['file'])) {
                        $image_in_backup .= basename($meta['meta']['file']);
                        $backup_file_exists = file_exists($image_in_backup);
                    } else {

                        $wp_meta = wp_get_attachment_metadata($id);

                        if (isset($wp_meta['file'])) {
                            $image_in_backup .= basename($wp_meta['file']);
                            $backup_file_exists = file_exists($image_in_backup);
                        }
                    }
                }

                $image_url = wp_get_attachment_url($id);
                $filename = basename($image_url);


                $image_path = get_attached_file($id);
                if (exif_imagetype($image_path) != IMAGETYPE_JPEG) {
                    return "";
                }
//.check if image already has meta data,
                $upload_dir = wp_upload_dir();
                $image_basedir = $upload_dir['basedir'];
                $array = explode("uploads", $image_url);

                // $image_path = $image_basedir . "" . $array[1];

                if (file_exists($image_path)) {
                    $exif = exif_read_data($image_path, 0, true);   //array reverse so that we get correct image title
                    $exif = array_reverse($exif);
                    foreach ($exif as $key => $section) {
                        foreach ($section as $name => $val) {
                 // print_r("$name: $val<br />\n") ;
                            if ($name == "UserComment") {
                                $image_title = $val;
                            }
                            if ($name == "ImageDescription") {
                                $image_desc = $val;
                            }
                        }
                    }
                    global $wpdb;
                    $image_addr = $wpdb->get_row("select * from " . $wpdb->prefix . "image_compression_details where image_id='" . $id . "'", ARRAY_A);
                    $address = $image_addr['address'];
                    if (!empty($image_title)) {
                        echo '<button style="padding-bottom: 2px;" onclick="' . $this->add_geo_data($id, $image_title, $image_desc, $address) . '" class="wpimage_req_btn" id="modal-link' . $id . '" data-id="' . $id . '" ><img style="width: 10px;" src="' . esc_url(plugin_dir_url(dirname(__FILE__)) . 'images/map-pin.png') . '" alt="" /></button>';
                    } else if (empty($image_title)) {
                        $image_title = "";
                        $image_desc = "";
                        $address = "";
                        echo '<button onclick="' . $this->add_geo_data($id, $image_title, $image_desc, $address) . '" class="wpimage_req_btn" id="modal-link' . $id . '" data-id="' . $id . '" > GEO TAG</button>';
                    } else {  //if there is no efix data yet linked to image
                        $image_title = "";
                        $image_desc = "";
                        $address = "";
                        echo '<button onclick="' . $this->add_geo_data($id, $image_title, $image_desc, $address) . '" class="wpimage_req_btn" id="modal-link' . $id . '" data-id="' . $id . '" > GEO TAG</button>';
                    }
                } else {
                    echo "NF";
                }


            } else {
                echo 'n/a';
            }
        } else if (0 === strcmp($column_name, 'quality')) {

            if (wp_attachment_is_image($id)) {

                $image_url = wp_get_attachment_url($id);
                $filename = basename($image_url);
                $image_path = get_attached_file($id);

                /*$publitio_api = new PublitioAPI('hxr7aqQDXG6WyLMApSjc', 'SmSt4vSBRtBW2m0kLAx5HsPikzhNwLuj');
                // list files
                $response = $publitio_api->call("/files/list", "GET", array('offset' => '0', 'limit' => '10'));
                // show file with id mytestfile
                $response = $publitio_api->call("/files/show/uXG6mDoy", "GET"); 
                $img = json_decode($response);

                debug($img); die();

                $img_src = $img->url_preview;
                $img_download_src = $img->url_download;

                echo'<img src="'.$img_src.'" width="100px">';

                echo'<a href="'.$img_download_src.'">Download file</a>';

                // create (upload) local file, simple
                $response = $publitio_api->upload_file($image_path, "file", 
                    array('name' => $filename, 
                          'position' => 'top-right', 
                          'padding' => '20')
                ); 

                // delete file with id mytestfile
                $response = $publitio_api->call("/files/delete/mytestfile", "DELETE"); 
*/


  
                /*echo "<img src='http://wp.test:8090/wp-content/uploads/2018/09/rabbit.jpg' alt='".$filename."'  width='100px'>";
                echo $id;

                echo '<select class="image_quality" name="image_quality">
                            <option value="auto">Auto</option>
                            <option value="jpg">JPG</option>
                            <option value="webp">WebP</option>
                        </select>
                        <button class="myajax"  data-action="keep_original" data-image_id="'.$id.'" data-image_path="'.$image_path.'" data-image_name="'.$filename.'"> Get Image </button>
                        <button class="myajax" data-action="replace_original" data-image_id="'.$id.'" data-image_path="'.$image_path.'" data-image_name="'.$filename.'"> Replace Image </button>';  */ 

            }
            
        }
    }


////////////////////////////////////////
// function my_custom_upload_mimes($mimes = array()) {

//     // Add a key and value for the CSV file type
//     $mimes['webp'] = "image/webp";
//     return $mimes;
// }

// add_action('upload_mimes', 'my_custom_upload_mimes');
    ////////////////////////////////////////


    public function gps2Num($coordPart)
    {
        $parts = explode('/', $coordPart);
        if (count($parts) <= 0)
            return 0;
        if (count($parts) == 1)
            return $parts[0];
        return floatval($parts[0]) / floatval($parts[1]);
    }

    public function getAddress($latitude, $longitude)
    {
        if (!empty($latitude) && !empty($longitude)) {
        //Send request and receive json data by address
            $geocodeFromLatLong = file_get_contents('http://maps.googleapis.com/maps/api/geocode/json?latlng=' . trim($latitude) . ',' . trim($longitude) . '&sensor=false');
            $output = json_decode($geocodeFromLatLong);
            $status = $output->status;
        //Get address from json data
            $address = ($status == "OK") ? $output->results[1]->formatted_address : '';
        //Return address of the given latitude and longitude
            if (!empty($address)) {
                return $address;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }



    public function add_geo_data($id, $image_title, $image_desc, $address)
    {
        echo '<div class="modal-wrapper">';
        echo '<div class="modal" id="modal' . $id . '">';
        echo '<div style="margin-left:95%"><button class="close-modal" id="closemodal' . $id . '" >X</button></div>';
        echo '<div class="modal-content"></div>';
        echo '</div>';
        echo '</div>';

        echo '<div class="show-in-modal" id="show-in-modal' . $id . '">';

        echo '</div>';
        echo '<input class="image_title" hidden value="' . stripslashes($image_title) . '">';
        echo '<textarea class="image_description" hidden>' . stripslashes($image_desc) . '</textarea>';

        // $arr =  json_encode(['id'=> $id, 'title'=> trim(preg_replace('/\s\s+/', ' ', $image_title)) , 'description'=> trim(preg_replace('/\s\s+/', ' ', $image_desc)) ]);
        ?>

<script>
jQuery(document).ready(function($) {

  var imageid = '<?php echo $id; ?>';

  // Close modal
  $('#closemodal'+imageid).click(function(e) {
    e.preventDefault(); // prevent link from being followed
    $("#modal"+imageid).toggleClass('show');
  });

  // Detect windows width function
  var $window = $(window);

  function checkWidth() {
    var windowsize = $window.width();
    if (windowsize > 767) {
      // if the window is greater than 767px wide then do below. we don't want the modal to show on mobile devices and instead the link will be followed.

      $("#modal-link"+imageid).click(function(e) {

        var image_title = jQuery(this).siblings('.image_title').val();
        var image_desc = jQuery(this).siblings('.image_description').text();

        var modalContent = $(".modal-content");
        var post_link =  '<form action="<?php echo esc_url(admin_url('admin.php?page=optimisationio-dashboard')); ?>" method="post">'+
                        '<input type="hidden" name="imageid" value="<?php echo $id ?>">'+
                        '<div class="form">'+
                            '<div class="input-group">'+
                                '<label for="">Title/ Custom Text:</label>'+
                                '<input name="image_title" value="'+image_title+'" type="text" required=""/>'+
                            '</div>'+
                            '<div class="input-group">'+
                                '<label for="">Image Description:</label>'+
                                '<textarea name="image_desc" class="text-large" rows="5">'+image_desc+'</textarea>'+
                            '</div>'+
                            '<div class="input-group">'+
                                '<label for="">Image Location:</label>'+
                                // '<input name="image_location" value="<?php echo $address ?>" type="text" required="">'+


                                '<div class="typeahead__container">'+
                                    '<div class="typeahead__field">'+
                                        '<div class="typeahead__query">'+
                                            '<input class="js-typeahead-country_v1  image_location" value="<?php echo $address ?>" id="image_location'+imageid+'" name="image_location" type="search" placeholder="Search"  autocomplete="off" required="">'+
                                        '</div>'+
                                    '</div>'+
                                '</div>'+

                                 '<input name="image_lat" class="image_lat" id="image_lat'+imageid+'" hidden>'+
                                 '<input name="image_lng" class="image_lng" id="image_lng'+imageid+'" hidden>'+

                            '</div>'+
                            '<div class="input-group">'+
                                '<button type="submit" > Update Image </button>'+
                            '</div>'+
                        '</div>'+
                    '</form>';
        e.preventDefault(); // prevent link from being followed

        $("#modal"+imageid).addClass('show', 1000, "easeOutSine");
        modalContent.empty();
        modalContent.append(post_link);

///////////////////////////////////////////////////////////////////////////
 $.typeahead({
                input: '.image_location',
                minLength: 2,
                order: "asc",
                dynamic: true,
                filter: false,
                delay: 1000,
                // cache: true,
                source: {
                    location: {
                        ajax: function (query) {
                            return {
                                type: "POST",
                                url: ajaxurl,
                                 // url: "http://data.campaigns.io/location/search/{{query}}",
                                 // dataType: 'jsonp',
                                 path: "data.location",
                                 data: {
                                    action: 'get_location',
                                    query: "{{query}}",
                                  }
                             }
                        }
                    },
                },
                callback: {
                    onClick: function (node, a, item, event) {
                        $(".image_lat").val(item.latitude);
                        $(".image_lng").val(item.longitude);
                    },
                    onCancel (node, event) {
                        $(".image_lat").val("");
                        $(".image_lng").val("");
                    }
                }
            });

        $(".image_location").keyup(function() {
            var term = jQuery(this).val();
            if( !term ) {
                $(".image_lat").val("");
                $(".image_lng").val("");
            }

        });
///////////////////////////////////////////////////////////////////

        return false;

      });
    }
  };

  checkWidth(); // excute function to check width on load
  $(window).resize(checkWidth); // execute function to check width on resize
});

</script>
<?php

}


/*public function get_lat_long($thisaddress)  // function to get the address
{
    $array = array();
    //use standard google api
    $geo = file_get_contents('https://maps.googleapis.com/maps/api/geocode/json?key=AIzaSyA5XZkBBIGesYlCCVkUeyBoiRc_IK0drEo&address=' . urlencode($thisaddress) . '&sensor=false');
       // We convert the JSON to an array
    $geo = json_decode($geo, true);
       // If everything is cool
    if ($geo['status'] = 'OK') {
        $latitude = @$geo['results'][0]['geometry']['location']['lat'];
        $longitude = @$geo['results'][0]['geometry']['location']['lng'];

        if(@$geo['results'][0]['partial_match']){
            $thisaddress = @$geo['results'][0]['formatted_address'];
        }

        $array = array('lat' => $latitude, 'lng' => $longitude ,'label' => $thisaddress);
        if (empty($array['lat'])) {
            //use our own api
            $geo = file_get_contents('https://geocoder.cit.api.here.com/6.2/geocode.json?app_id=lUpWQzpnGKl9lGnfVS04&app_code=HN39Z423n1y4zSD8K7slGg&gen=8&searchtext=' . urlencode($thisaddress) . '&key=AIzaSyA5XZkBBIGesYlCCVkUeyBoiRc_IK0drEo');
            $geo = json_decode($geo, true);
            if (!empty($geo['Response']['View'])) {
                $latitude = @$geo['Response']['View'][0]['Result'][0]['Location']['NavigationPosition'][0]['Latitude'];
                $longitude = @$geo['Response']['View'][0]['Result'][0]['Location']['NavigationPosition'][0]['Longitude'];
                $locationname = @$geo['Response']['View'][0]['Result'][0]['Location']['Address']['Label'];
                $array = array('lat' => $latitude, 'lng' => $longitude,'label'=> $locationname);
                if (empty($array['lat'])) {
                }
            }
        }
    }

    return $array;
}*/

public function save_meta_data($imageId, $image_url, $filename, $title, $description, $longitude, $latitude)
{
// image gps info
    $upload_dir = wp_upload_dir();
    $image_basedir = $upload_dir['basedir'];
    $array = explode("uploads", $image_url);
    $image_path = $image_basedir . "" . $array[1];
    $input = $image_path;
    $output = $image_path;
    $comment = $title;
    $model = "";
    $altitude = '';
    $date_time = date("Y-m-d H:i:s");
    $this->addGpsInfo($input, $output, $description, $comment, $model, $longitude, $latitude, $altitude, $date_time);
    $sizes = get_intermediate_image_sizes();
    foreach ($sizes as $key => $size) {
        $thumb = wp_get_attachment_image_src($imageId, $size);
        $this_image_url = $thumb[0];
        $array = explode("uploads", $this_image_url);
        $this_image_path = $image_basedir . "" . $array[1];
        $input = $this_image_path;
        $output = $this_image_path;
        if (file_exists($input)) {
            $this->addGpsInfo($input, $output, $description, $comment, $model, $longitude, $latitude, $altitude, $date_time);
        } else {
            echo "<br> Size No file " . $input;
        }
    }

}

public function addGpsInfo($input, $output, $description, $comment, $model, $longitude, $latitude, $altitude, $date_time)
{
    $jpeg = new PelJpeg($input);
    $exif = new PelExif();
    $jpeg->setExif($exif);
    $tiff = new PelTiff();
    $exif->setTiff($tiff);
    $ifd0 = new PelIfd(PelIfd::IFD0);
    $tiff->setIfd($ifd0);
    $gps_ifd = new PelIfd(PelIfd::GPS);
    $ifd0->addSubIfd($gps_ifd);
    $exif_ifd = new PelIfd(PelIfd::EXIF);
    $exif_ifd->addEntry(new PelEntryUserComment($comment));
    $ifd0->addSubIfd($exif_ifd);

    $inter_ifd = new PelIfd(PelIfd::INTEROPERABILITY);
    $ifd0->addSubIfd($inter_ifd);

    $ifd0->addEntry(new PelEntryAscii(PelTag::MODEL, $model));
    $ifd0->addEntry(new PelEntryAscii(PelTag::DATE_TIME, $date_time));
    $ifd0->addEntry(new PelEntryAscii(PelTag::IMAGE_DESCRIPTION, $description));

    $gps_ifd->addEntry(new PelEntryByte(PelTag::GPS_VERSION_ID, 2, 2, 0, 0));

    list($hours, $minutes, $seconds) = $this->convertDecimalToDMS($latitude);

    $latitude_ref = ($latitude < 0) ? 'S' : 'N';

    $gps_ifd->addEntry(new PelEntryAscii(PelTag::GPS_LATITUDE_REF, $latitude_ref));
    $gps_ifd->addEntry(new PelEntryRational(PelTag::GPS_LATITUDE, $hours, $minutes, $seconds));

    /* The longitude works like the latitude. */
    list($hours, $minutes, $seconds) = $this->convertDecimalToDMS($longitude);
    $longitude_ref = ($longitude < 0) ? 'W' : 'E';

    $gps_ifd->addEntry(new PelEntryAscii(PelTag::GPS_LONGITUDE_REF, $longitude_ref));
    $gps_ifd->addEntry(new PelEntryRational(PelTag::GPS_LONGITUDE, $hours, $minutes, $seconds));
    if ($altitude) {


        $gps_ifd->addEntry(new PelEntryRational(PelTag::GPS_ALTITUDE, array(
            abs($altitude),
            1
        )));
    }
    $gps_ifd->addEntry(new PelEntryByte(PelTag::GPS_ALTITUDE_REF, (int)($altitude < 0)));

    /* Finally we store the data in the output file. */
    file_put_contents($output, $jpeg->getBytes());
}
public function convertDecimalToDMS($degree)
{
    if ($degree > 180 || $degree < -180) {
        return null;
    }
    $degree = abs($degree);
    $seconds = $degree * 3600; // Total number of seconds.
    $degrees = floor($degree); // Number of whole degrees.
    $seconds -= $degrees * 3600; // Subtract the number of seconds// taken by the degrees.
    $minutes = floor($seconds / 60); // Number of whole minutes.
    $seconds -= $minutes * 60; // Subtract the number of seconds // taken by the minutes.
    $seconds = round($seconds * 100, 0); // Round seconds with a 1/100th  // second precision.

    return array(
        array($degrees, 1),
        array($minutes, 1),
        array($seconds, 100)
    );
}

public function replace_image($image_path, $compressed_url)
{
    return wpimages_replace_media_image($image_path, $compressed_url);
}

public function optimize_image($image_path, $backup)
{
    return wpimages_optimize_image($image_path, $backup);
}

public function delete_resources($public_id)
{
    $settings = $this->compression_settings;
    $wpimage = new Wpimage();
    $wpimage->deleteResource($public_id);
}

public function optimize_thumbnails($image_data)
{

    $image_id = $this->id;

    if (empty($image_id)) {
        global $wpdb;
        $post = $wpdb->get_row($wpdb->prepare("SELECT post_id FROM $wpdb->postmeta WHERE meta_value = %s LIMIT 1", $image_data['file']));
        $image_id = $post->post_id;
    }

    $path_parts = pathinfo($image_data['file']);

        // e.g. 04/02, for use in getting correct path or URL
    $upload_subdir = $path_parts['dirname'];

    $upload_dir = wp_upload_dir();

        // all the way up to /uploads
    $upload_base_path = $upload_dir['basedir'];
    $upload_full_path = $upload_base_path . '/' . $upload_subdir;

    $sizes = array();

    if (isset($image_data['sizes'])) {
        $sizes = $image_data['sizes'];
    }

    if (!empty($sizes)) {

        $thumb_path = '';

        $thumbs_optimized_store = array();
        $this_thumb = array();

        foreach ($sizes as $key => $size) {

            $thumb_path = $upload_full_path . '/' . $size['file'];

            if (file_exists($thumb_path) !== false) {

                $result = $this->optimize_image($thumb_path, $this->backup_before_compression);

                if (!empty($result) && isset($result) && isset($result['url'])) {
                    $compressed_url = $result["url"];
                    if ($this->replace_image($thumb_path, $compressed_url)) {
                        $this_thumb = array('thumb' => $key, 'file' => $size['file'], 'original_size' => $result['original_size'], 'compressed_size' => $result['bytes'], 'backup_before_compression' => $this->backup_before_compression);
                        $thumbs_optimized_store[] = $this_thumb;
                    }
                        //deleting the image from clould
                    $this->delete_resources($result['public_id']);
                }
            }
        }
    }
    if (!empty($thumbs_optimized_store)) {
        update_post_meta($image_id, '_compressed_thumbs', $thumbs_optimized_store, false);
    }
    return $image_data;
}

public function exe_before_wpheader()
{

    if (is_admin() && current_user_can('manage_options')) {

        $server = $_SERVER;   // Input var okay.
        $post_req = 'post' === sanitize_key($server['REQUEST_METHOD']) ? $_POST : array();   // Input var okay.

        //GEO TAG btn
        if (isset($_POST['image_title'])) {

            $imageId = $_POST['imageid'];
            $image_url = wp_get_attachment_url($imageId);
            $filename = basename($image_url);
            $title = $_POST['image_title'];
            $description = $_POST['image_desc'];
            $thisaddress = $_POST['image_location'];

            // save address in database so that address can be fetched easily on editing image
            global $wpdb;
            $image = $wpdb->get_row("select * from " . $wpdb->prefix . "image_compression_details where image_id='" . $imageId . "'", ARRAY_A);


            //update image
            if (!empty($image)) {
                    // if address already exists in our db, get lat, lng and save location, no need to get lat, lng
                    // if (!empty($_POST['image_lat'])) {
                        if (!empty($_POST['image_lat'])) {
                            $latitude = $_POST['image_lat'];
                            $longitude = $_POST['image_lng'];
                        } else {
                            $latitude = $image['latitude'];
                            $longitude = $image['longitude'];
                        }
                        //update address to fetch on update from image_compression table
                        $data['address'] = $thisaddress;
                        $data['file_title'] = $title;
                        $data['file_description'] = $description;
                        $data['latitude'] = $latitude;
                        $data['longitude'] = $longitude;

                        $wpdb->update($wpdb->prefix . "image_compression_details", $data, array('image_id' => $imageId));

                        $this->save_meta_data($imageId, $image_url, $filename, $title, $description, $longitude, $latitude);
                  /*  } else {
                        $this->get_and_save_latlng($imageId, $image_url, $filename, $title, $description, $thisaddress);
                    }*/
            //insert image
            } else {
                if (!empty($_POST['image_lat'])) {
                    $latitude = $_POST['image_lat'];
                    $longitude = $_POST['image_lng'];

                    //save address to fetch on update from image_compression table
                    $file_data['image_id'] = $imageId;
                    $file_data['address'] = $thisaddress;
                    $file_data['file_title'] = $title;
                    $file_data['file_description'] = $description;
                    $file_data['latitude'] = $latitude;
                    $file_data['longitude'] = $longitude;
                    $wpdb->insert($wpdb->prefix . "image_compression_details", $file_data);

                    $this->save_meta_data($imageId, $image_url, $filename, $title, $description, $longitude, $latitude);
                } /*else {
                    $this->get_and_save_latlng($imageId, $image_url, $filename, $title, $description, $thisaddress);
                }*/
            }
            wp_redirect(get_site_url() . "/wp-admin/upload.php");
            die("---");

        }

        if (isset($post_req['optimisation_save_image_compression_settings'])) {   // NOTE:Save ONLY on form submit. Deny access from another call (eg. AJAX).

            if (isset($post_req['optimisationio_image_compression_settings']) && wp_verify_nonce($post_req['optimisationio_image_compression_settings'], 'optimisationio-image-compression-settings')) {

                    // Image Compression settings.
                update_option('wpimages_max_height', isset($post_req['wpimages_max_height']) ? $post_req['wpimages_max_height'] : WPIMAGE_DEFAULT_MAX_HEIGHT);
                update_option('wpimages_max_width', isset($post_req['wpimages_max_width']) ? $post_req['wpimages_max_width'] : WPIMAGE_DEFAULT_MAX_WIDTH);
                update_option('wpimages_max_height_library', isset($post_req['wpimages_max_height_library']) ? $post_req['wpimages_max_height_library'] : WPIMAGE_DEFAULT_MAX_WIDTH);
                update_option('wpimages_max_width_library', isset($post_req['wpimages_max_width_library']) ? $post_req['wpimages_max_width_library'] : WPIMAGE_DEFAULT_MAX_WIDTH);
                update_option('wpimages_max_height_other', isset($post_req['wpimages_max_height_other']) ? $post_req['wpimages_max_height_other'] : WPIMAGE_DEFAULT_MAX_WIDTH);
                update_option('wpimages_max_width_other', isset($post_req['wpimages_max_width_other']) ? $post_req['wpimages_max_width_other'] : WPIMAGE_DEFAULT_MAX_WIDTH);
                update_option('wpimages_bmp_to_jpg', isset($post_req['wpimages_bmp_to_jpg']) && 1 === (int)$post_req['wpimages_bmp_to_jpg'] ? 1 : 0);
                update_option('wpimages_png_to_jpg', isset($post_req['wpimages_png_to_jpg']) && 1 === (int)$post_req['wpimages_png_to_jpg'] ? 1 : 0);
                update_option('wpimages_quality', isset($post_req['wpimages_quality']) ? $post_req['wpimages_quality'] : WPIMAGE_DEFAULT_QUALITY);
                update_option('wpimages_quality_auto', isset($post_req['wpimages_quality_auto']) ? $post_req['wpimages_quality_auto'] : 'auto');

                    // Lazy Load settings.
                $lazy_load_values = serialize($post_req['_wpimage_lazyload']);
                update_option('_wpimage_lazyload_options', $lazy_load_values);
            }
        }

        if (isset($post_req['optimisation_save_cloudinary_api_settings'])) {   // NOTE:Save ONLY on form submit. Deny access from another call (eg. AJAX).

            if (isset($post_req['optimisationio_cloudinary_api_settings']) && wp_verify_nonce($post_req['optimisationio_cloudinary_api_settings'], 'optimisationio-cloudinary-api-settings')) {

                $enabled = isset($post_req['custom_cloudinary']['enabled']) ? $post_req['custom_cloudinary']['enabled'] : 0;

                $options = array(
                    'invalid_msg' => '',
                    'success_msg' => '',
                    'cloud_name' => isset($post_req['custom_cloudinary']['cloud_name']) ? sanitize_text_field($post_req['custom_cloudinary']['cloud_name']) : '',
                    'api_key' => isset($post_req['custom_cloudinary']['api_key']) ? sanitize_text_field($post_req['custom_cloudinary']['api_key']) : '',
                    'api_secret' => isset($post_req['custom_cloudinary']['api_secret']) ? sanitize_text_field($post_req['custom_cloudinary']['api_secret']) : '',
                );

                if ($enabled) {
                    $valid = $this->validate_cloudinary_credentials($options['cloud_name'], $options['api_key'], $options['api_secret']);
                    $options['enabled'] = $enabled && $valid ? 1 : 0;
                    $options['invalid_msg'] = !$valid ? __('Invalid Cloudinary credentials', 'wpimage') : '';
                    $options['success_msg'] = $valid ? __('Cloudinary credentials validated and saved successful', 'wpimage') : '';
                } else {
                    $options['enabled'] = 0;
                }

                update_option('_wpimage_options_cloudinary', $options);

                update_option('_wpimage_options', serialize(isset($post_req['_wpimage_options']) ? $post_req['_wpimage_options'] : array()));
            }
        }

    }
}

/*public function get_and_save_latlng($imageId, $image_url, $filename, $title, $description, $thisaddress)
{
    $latlong = $this->get_lat_long($thisaddress);
    if (isset($latlong['lng'])) {
        $longitude = sprintf("%.6f", $latlong['lng']);
        $latitude = sprintf("%.6f", $latlong['lat']);
        if(isset($latlong['label'])){
            $thisaddress= $latlong['label'];
        }
            //insert and update image only if lat and lng for image are found
            //get image
        global $wpdb;
        $image = $wpdb->get_row("select * from " . $wpdb->prefix . "image_compression_details where image_id='" . $imageId . "'", ARRAY_A);
        if (empty($image)) {    //insert
            $file_data['image_id'] = $imageId;
            $file_data['address'] = $thisaddress;
            $file_data['file_title'] = $title;
            $file_data['file_description'] = $description;
            $wpdb->insert($wpdb->prefix . "image_compression_details", $file_data);
        }
        if (!empty($image)) {  //update
            $data['address'] = $thisaddress;
            $data['file_title'] = $title;
            $data['file_description'] = $description;
            $wpdb->update($wpdb->prefix . "image_compression_details", $data, array('image_id' => $imageId));
        }
            // for both insert and update, if it comes into this function, i.e., if address is changed or new image
        $data['longitude'] = $longitude;
        $data['latitude'] = $latitude;
        $wpdb->update($wpdb->prefix . "image_compression_details", $data, array('image_id' => $imageId));

        $this->save_meta_data($imageId, $image_url, $filename, $title, $description, $longitude, $latitude);

    }
}*/

public static function pretty_kb($bytes)
{
    return wpimages_pretty_kb($bytes);
}

public static function addon_settings()
{
    include dirname(dirname(__FILE__)) . '/views/dashboard-settings.php';
}

public static function cloudinary_settings($include_upgrade = true)
{
    include dirname(dirname(__FILE__)) . '/views/cloudinary-settings.php';
}
}




add_action( 'wp_ajax_get_location', 'get_location' );

function get_location() {
    global $wpdb; // this is how you get access to the database

    $term = $_POST['query'];

    $url = "http://data.campaigns.io/location/search/".$term;

    $response = wp_remote_get( $url );
    if ( is_array( $response ) ) {
      $header = $response['headers']; // array of http header lines
      $body = $response['body']; // use the content
    }
    echo $body;

    wp_die(); // this is required to terminate immediately and return a proper response
}







/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
add_action('admin_head', 'my_action_javascript');

function my_action_javascript() {
?>
<script type="text/javascript" >
jQuery(document).ready(function($) {

    $('.myajax').click(function(){

        $image_path = $(this).data('image_path');
        $image_name = $(this).data('image_name');
        $image_quality = $(this).siblings('.image_quality').val();
        $image_id = $(this).data('image_id');

        $image_action = $(this).data('action');

        var data = {
            action: 'my_action',
            image_path: $image_path,
            image_quality: $image_quality,
            image_name: $image_name,
            image_id: $image_id,
            image_action: $image_action
        };

        //my_action_callback
        $.post(ajaxurl, data, function(response) {
            alert(response);
        });
    });


});
</script>
<?php
}

add_action('wp_ajax_my_action', 'my_action');

function my_action() {

   
/*    global $wpdb; // this is how you get access to the database
    $image_id = $_POST['image_id'];
    $image_path = $_POST['image_path'];
    $image_name = $_POST['image_name'];

        $extensions = array('.jpg', '.JPG', '.png' ,'.PNG' ,'.jpeg' ,'.JPEG');
        foreach ($extensions as $key => $extension) {
            if (strpos($image_name, $extension) !== false) {
                $img_name = str_replace($extension,"",$image_name);
            }
        }
     
    $image_quality = $_POST['image_quality'];
    $image_action = $_POST['image_action'];
//check for multiple same image upload
//once an image gets uploaded in any format to the cloud, it just needs to be fetched below
    //first upload this image on cloud
    // \Cloudinary\Uploader::upload($image_path, array("public_id" => $img_name));
    // cloudinary_url($img_name.".jpg");
    // http://res.cloudinary.com/cdn-hosting-io/image/upload/cat.jpg

    $custom_cloudinary_options = get_option('_wpimage_options_cloudinary');
    if (!empty($custom_cloudinary_options)) {
       $cloud_name = $custom_cloudinary_options['cloud_name'];
    } else {
       $cloud_name = 'cdn-hosting-io';
    }
    
    if ($image_quality == 'jpg') {

        $format = '.jpg';

        $get_image_src = cl_image_tag($img_name."".$format, array("cloud_name" => $cloud_name, "width"=>150, "height"=>100));
    } else if ($image_quality == 'webp') {

        $format = '.webp';
        $get_image_src = cl_image_tag($img_name."".$format, array("cloud_name" => $cloud_name, "width"=>150, "height"=>100));

    } else if ($image_quality == 'auto') {

        $get_image_src = cl_image_tag($image_name, array("cloud_name" => $cloud_name, "width"=>350, "fetch_format"=>"auto", "crop"=>"scale"));
    }

    $img_path =  str_replace($image_name,"",$image_path);
    $newimgname = $img_name."".$format;

    $old_file_data = get_post_meta( $image_id, '_wp_attached_file' );
    $old_file_attachment_metadata = get_post_meta( $image_id, '_wp_attachment_metadata' );

    $a = explode("uploads/",$image_path);
    $b = $a[1];
    $YYYYMM = str_replace($image_name,"",$b);

//replace previous file _wp_attached_file key with this new file's _wp_attached_file key
    // update_post_meta( $image_id, '_wp_attached_file', $YYYYMM."".$newimgname, $YYYYMM."".$image_name );  //2018/09/rabbit.jpg

//replace previous file _wp_attachment_metadata key with this new file's _wp_attachment_metadata key
    // update_post_meta( $image_id, '_wp_attachment_metadata', $YYYYMM."".$newimgname, $YYYYMM."".$image_name );  //big array

/////////////////////////////////////////////////////////////////////////////////
//changes made to wp-config.php for .webp file type upload 
    $get_image_src = "http://res.cloudinary.com/cdn-hosting-io/image/upload/cat.webp";

    echo $get_image_src; 
    if ( !function_exists('media_handle_upload') ) {
        require_once(ABSPATH . "wp-admin" . '/includes/image.php');
        require_once(ABSPATH . "wp-admin" . '/includes/file.php');
        require_once(ABSPATH . "wp-admin" . '/includes/media.php');
    }

    $url = $get_image_src;
    $tmp = download_url( $url );
    if( is_wp_error( $tmp ) ){
        // download failed, handle error
    }
    $post_id = $image_id;
    $desc = $newimgname;
    $file_array = array();

    // Set variables for storage
    // fix file filename for query strings
    preg_match('/[^\?]+\.(jpg|jpe|jpeg|gif|png|webp)/i', $url, $matches);
    $file_array['name'] = basename($matches[0]);
    $file_array['tmp_name'] = $tmp;

    // If error storing temporarily, unlink
    if ( is_wp_error( $tmp ) ) {
        @unlink($file_array['tmp_name']);
        $file_array['tmp_name'] = '';
    }

    // do the validation and storage stuff
    $new_imageId = media_handle_sideload( $file_array, $post_id, $desc );
    // $new_file_data = get_post_meta( $new_imageId, '_wp_attached_file' );
    // $new_file_attachment_metadata = get_post_meta( $new_imageId, '_wp_attachment_metadata' );

    // If error storing permanently, unlink
    if ( is_wp_error($new_imageId) ) {
        @unlink($file_array['tmp_name']);
        return $new_imageId;
    }
 //////////////////////////////////////////////////////////////////

//on Replace existing Image
    // get meta data saved for old image from our db, saved when image was updated or inserted
if ($new_imageId != null && $image_action == 'replace_original') {
    
    global $wpdb;
    $old_imageId = $image_id;
    $old_image = $wpdb->get_row("select * from " . $wpdb->prefix . "image_compression_details where image_id='" . $old_imageId . "'", ARRAY_A);

        $imageId = $new_imageId;
        $image_url = wp_get_attachment_url($imageId);
        $filename = basename($image_url);
        $title = $old_image['file_title'];
        $description = $old_image['file_description'];
        $latitude = $old_image['latitude'];
        $longitude = $old_image['longitude'];

        $this->save_meta_data($imageId, $image_url, $filename, $title, $description, $longitude, $latitude);

    //insert meta data of this new image in our db for this image can be used again to fetch a new format image
        $file_data['image_id'] = $new_imageId;
        $file_data['address'] = $old_image['address'];
        $file_data['file_title'] = $old_image['file_title'];
        $file_data['file_description'] = $old_image['file_description'];
        $file_data['latitude'] = $old_image['latitude'];
        $file_data['longitude'] = $old_image['longitude'];
        $wpdb->insert($wpdb->prefix . "image_compression_details", $file_data);

        //delete old image from wordpress
        // delete_post_meta($post_id);
        wp_delete_attachment( $post_id );

}*/

///////////////////////////

    exit(); 
}