<?php
/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * Dashboard. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://optimisation.io
 * @since             1.0.0

 * Copyright (C) 2018 hosting.io
 *
 * @wordpress-plugin
 * Plugin Name:       JPG, PNG Compression and Optimization
 * Plugin URI:        https://hosting.io
 * Description:       Image Compression and resizing - Setup under the Tools menu
 * Version:           1.7.31
 * Author:            pigeonhut, https://hosting.io, Jody Nesbitt
 * Author URI:        https://hosting.io
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 */

define("OPTIMISATIONIO_IMAGE_COMPRESSION_ADDON", true);

define('WPIMAGE_VERSION', '1.6.28');
define('WPIMAGE_SCHEMA_VERSION', '1.1');

define('WPIMAGE_DEFAULT_MAX_WIDTH', 1024);
define('WPIMAGE_DEFAULT_MAX_HEIGHT', 1024);
define('WPIMAGE_DEFAULT_BMP_TO_JPG', 1);
define('WPIMAGE_DEFAULT_PNG_TO_JPG', 0);
define('WPIMAGE_DEFAULT_QUALITY', 90);

define('WPIMAGE_SOURCE_POST', 1);
define('WPIMAGE_SOURCE_LIBRARY', 2);
define('WPIMAGE_SOURCE_OTHER', 4);

define('WPIMAGE_AJAX_MAX_RECORDS', 265);
define('STRIPE_APIKEY', 'sk_live_bD9bw5p3KkcWtmAB9e5hhJb2');

$wpImagelazySizesDefaults = serialize( array(
    'lazyload_expand' => 359,
    'lazyload_optimumx' => 'false',
    'lazyload_intrinsicRatio' => 'false',
    'lazyload_iframe' => 'false',
    'lazyload_autosize' => 'true',
    'lazyload_preloadAfterLoad' => 'false'
) );

require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

load_plugin_textdomain('wp-image-compression', false, 'wp-image-compression/languages/');
register_activation_hook(__FILE__, 'wpimagecompression_install');

function wpimagecompression_install() {
    global $wpdb;
    $table_name = $wpdb->prefix . "image_compression_settings";
    $sqle = "DROP TABLE IF EXISTS $table_name;";
    $sql = "CREATE TABLE $table_name (
        `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
         `total_size_optimized` float DEFAULT NULL,
         `total_image_optimized` int(11) DEFAULT NULL,
         `total_allowed` int(11) NOT NULL DEFAULT '500',
         `created` datetime DEFAULT NULL,
         `modified` datetime DEFAULT NULL,
         PRIMARY KEY (`id`) );";
    dbDelta($sqle);
    dbDelta($sql);

    $table_name1 = $wpdb->prefix . "image_compression_details";
    $sqle1 = "DROP TABLE IF EXISTS $table_name1;";
    $sql1 = "CREATE TABLE $table_name1 (
        `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
         `image_id` int(11) DEFAULT NULL,
         `file_title` varchar(255) DEFAULT NULL,
         `file_description` varchar(500) DEFAULT NULL,
         `address` varchar(500) DEFAULT NULL,
         `longitude` varchar(100) DEFAULT NULL,
         `latitude` varchar(100) DEFAULT NULL,
         `created` datetime DEFAULT NULL,
         `modified` datetime DEFAULT NULL,
         PRIMARY KEY (`id`) );";
    dbDelta($sqle1);
    dbDelta($sql1);
}

/**
 * import supporting libraries
 */
include_once plugin_dir_path(__FILE__) . 'libs/utils.php';
include_once plugin_dir_path(__FILE__) . 'settings.php';
include_once plugin_dir_path(__FILE__) . 'ajax.php';

$stripe_dir = dirname(__FILE__) . '/lib/stripe/';

// Including stripe stuffs.

// Stripe singleton.
require( $stripe_dir . 'Stripe.php');

// Utilities
require( $stripe_dir . 'Util/AutoPagingIterator.php');
require( $stripe_dir . 'Util/RequestOptions.php');
require( $stripe_dir . 'Util/Set.php');
require( $stripe_dir . 'Util/Util.php');

// HttpClient.
require( $stripe_dir . 'HttpClient/ClientInterface.php');
require( $stripe_dir . 'HttpClient/CurlClient.php');

// Errors.
require( $stripe_dir . 'Error/Base.php');
require( $stripe_dir . 'Error/Api.php');
require( $stripe_dir . 'Error/ApiConnection.php');
require( $stripe_dir . 'Error/Authentication.php');
require( $stripe_dir . 'Error/Card.php');
require( $stripe_dir . 'Error/InvalidRequest.php');
require( $stripe_dir . 'Error/Permission.php');
require( $stripe_dir . 'Error/RateLimit.php');

// Plumbing.
require( $stripe_dir . 'ApiResponse.php');
require( $stripe_dir . 'JsonSerializable.php');
require( $stripe_dir . 'StripeObject.php');
require( $stripe_dir . 'ApiRequestor.php');
require( $stripe_dir . 'ApiResource.php');
require( $stripe_dir . 'SingletonApiResource.php');
require( $stripe_dir . 'AttachedObject.php');
require( $stripe_dir . 'ExternalAccount.php');

// Stripe API Resources.
require( $stripe_dir . 'Account.php');
require( $stripe_dir . 'AlipayAccount.php');
require( $stripe_dir . 'ApplePayDomain.php');
require( $stripe_dir . 'ApplicationFee.php');
require( $stripe_dir . 'ApplicationFeeRefund.php');
require( $stripe_dir . 'Balance.php');
require( $stripe_dir . 'BalanceTransaction.php');
require( $stripe_dir . 'BankAccount.php');
require( $stripe_dir . 'BitcoinReceiver.php');
require( $stripe_dir . 'BitcoinTransaction.php');
require( $stripe_dir . 'Card.php');
require( $stripe_dir . 'Charge.php');
require( $stripe_dir . 'Collection.php');
require( $stripe_dir . 'CountrySpec.php');
require( $stripe_dir . 'Coupon.php');
require( $stripe_dir . 'Customer.php');
require( $stripe_dir . 'Dispute.php');
require( $stripe_dir . 'Event.php');
require( $stripe_dir . 'FileUpload.php');
require( $stripe_dir . 'Invoice.php');
require( $stripe_dir . 'InvoiceItem.php');
require( $stripe_dir . 'Order.php');
require( $stripe_dir . 'OrderReturn.php');
require( $stripe_dir . 'Plan.php');
require( $stripe_dir . 'Product.php');
require( $stripe_dir . 'Recipient.php');
require( $stripe_dir . 'Refund.php');
require( $stripe_dir . 'SKU.php');
require( $stripe_dir . 'Source.php');
require( $stripe_dir . 'Subscription.php');
require( $stripe_dir . 'SubscriptionItem.php');
require( $stripe_dir . 'ThreeDSecure.php');
require( $stripe_dir . 'Token.php');
require( $stripe_dir . 'Transfer.php');
require( $stripe_dir . 'TransferReversal.php');

add_action( 'wp_ajax_stripe_payment', 'stripe_payment' );
add_action( 'wp_ajax_nopriv_stripe_payment', 'stripe_payment' );

add_action('admin_init', 'notify_imagecompression_upgrade');

function notify_imagecompression_upgrade() {
       global $wpdb;

       $custom_cloudinary_options = get_option('_wpimage_options_cloudinary');

       if( ! $custom_cloudinary_options || 1 !== (int) $custom_cloudinary_options['enabled'] ){

           $exist = $wpdb->get_row( "SELECT * FROM " . $wpdb->prefix . "image_compression_settings", ARRAY_A );

           if( $exist && 0 >= $exist['total_allowed'] ) {
                add_action('admin_notices', 'wp_image_compression_upgrade_notice');
           }
       }
       return true;
}

function wp_image_compression_upgrade_notice() {
    echo '<div class="error"><p><strong>';
    printf(
        'You have reached your free limit of 500 Images <a href="%2$s">%1$s</a> Please purchase a Compression plan to continue optimising',
        getPluginName(),
        get_admin_url()."tools.php?page=wp-convertor"
    );
    echo '</strong></p></div>';
}

function getPluginName() {
    $data = get_plugin_data(__FILE__);
    return $data['Name'];
}

/**
 * Fired with the WordPress upload dialog is displayed
 */

/**
 * Inspects the request and determines where the upload came from
 * @return WPIMAGE_SOURCE_POST | WPIMAGE_SOURCE_LIBRARY | WPIMAGE_SOURCE_OTHER
 */
function wpimages_get_source() {

    $id     = array_key_exists('post_id', $_REQUEST) ? $_REQUEST['post_id'] : '';
    $action = array_key_exists('action', $_REQUEST) ? $_REQUEST['action'] : '';

    // a post_id indicates image is attached to a post
    if ($id > 0) {
        return WPIMAGE_SOURCE_POST;
    }

    // post_id of 0 is 3.x otherwise use the action parameter
    if ($id === 0 || $action == 'upload-attachment') {
        return WPIMAGE_SOURCE_LIBRARY;
    }

    // we don't know where this one came from but $_REQUEST['_wp_http_referer'] may contain info
    return WPIMAGE_SOURCE_OTHER;
}

/**
 * Given the source, returns the max width/height
 *
 * @example:  list($w,$h) = wpimages_get_max_width_height(WPIMAGE_SOURCE_LIBRARY);
 * @param int WPIMAGE_SOURCE_POST | WPIMAGE_SOURCE_LIBRARY | WPIMAGE_SOURCE_OTHER
 */
function wpimages_get_max_width_height($source) {

    $w = wpimages_get_option('wpimages_max_width', WPIMAGE_DEFAULT_MAX_WIDTH);
    $h = wpimages_get_option('wpimages_max_height', WPIMAGE_DEFAULT_MAX_HEIGHT);

    switch ($source) {
        case WPIMAGE_SOURCE_POST:
            break;
        case WPIMAGE_SOURCE_LIBRARY:
            $w = wpimages_get_option('wpimages_max_width_library', $w);
            $h = wpimages_get_option('wpimages_max_height_library', $h);
            break;
        default:
            $w = wpimages_get_option('wpimages_max_width_other', $w);
            $h = wpimages_get_option('wpimages_max_height_other', $h);
            break;
    }

    return array($w, $h);
}

function wpimages_makedirs($dirpath, $mode=0777) {
    return is_dir($dirpath) || mkdir($dirpath, $mode, true);
}

/**
 * Handler after a file has been uploaded.  If the file is an image, check the size
 * to see if it is too big and, if so, resize and overwrite the original
 * @param Array $params
 */
function wpimages_handle_upload($params) {

    /* debug logging... */
    // file_put_contents ( "debug.txt" , print_r($params,1) . "\n" );
    // if "noresize" is included in the filename then we will bypass wpimage scaling
    if (strpos($params['file'], 'noresize') !== false) {
        return $params;
    }

    // if preferences specify so then we can convert an original bmp or png file into jpg
    if ( 'image/bmp' === $params['type'] && wpimages_get_option('wpimages_bmp_to_jpg', WPIMAGE_DEFAULT_BMP_TO_JPG)) {
        $params = wpimages_convert_to_jpg('bmp', $params);
    }

    if ( 'image/png' === $params['type'] && wpimages_get_option('wpimages_png_to_jpg', WPIMAGE_DEFAULT_PNG_TO_JPG)) {
        $params = wpimages_convert_to_jpg('png', $params);
    }

    // make sure this is a type of image that we want to convert and that it exists
    // @TODO when uploads occur via RPC the image may not exist at this location
    $oldPath = $params['file'];

    if ((!is_wp_error($params)) && file_exists($oldPath) && in_array($params['type'], array('image/png', 'image/gif', 'image/jpeg'))) {

        // figure out where the upload is coming from
        $source = wpimages_get_source();

        list($maxW, $maxH) = wpimages_get_max_width_height($source);

        list($oldW, $oldH) = getimagesize($oldPath);

        /* HACK: if getimagesize returns an incorrect value (sometimes due to bad EXIF data..?)
        $img = imagecreatefromjpeg ($oldPath);
        $oldW = imagesx ($img);
        $oldH = imagesy ($img);
        imagedestroy ($img);
        // */

        /* HACK: an animated gif may have different frame sizes.  to get the "screen" size
        $data = ''; // TODO: convert file to binary
        $header = unpack('@6/vwidth/vheight', $data );
        $oldW = $header['width'];
        $oldH = $header['width'];
        // */

        if (($oldW > $maxW && $maxW > 0) || ($oldH > $maxH && $maxH > 0)) {

            $quality = wpimages_get_option('wpimages_quality', WPIMAGE_DEFAULT_QUALITY);

            list($newW, $newH) = wp_constrain_dimensions($oldW, $oldH, $maxW, $maxH);

            // this is wordpress prior to 3.5 (image_resize deprecated as of 3.5)
            $resizeResult = wpimages_image_resize($oldPath, $newW, $newH, false, null, null, $quality);

            /* uncomment to debug error handling code: */
            // $resizeResult = new WP_Error('invalid_image', __(print_r($_REQUEST)), $oldPath);

            // regardless of success/fail we're going to remove the original upload
            unlink($oldPath);

            if (!is_wp_error($resizeResult)) {

                $newPath = $resizeResult;

                // remove original and replace with re-sized image
                rename($newPath, $oldPath);
            }
            else {
                // resize didn't work, likely because the image processing libraries are missing
                $params = wp_handle_upload_error( $oldPath, sprintf( __("Oh Snap! Wp image resizer was unable to resize this image "
                    . "for the following reason: '%s'.  If you continue to see this error message, you may need to upgrade plugin"
                    . " or disable the Wp image resizer plugin."
                    . "  If you think you have discovered a bug, please report it on the Wp image resizer support forum.", 'wpimage' ), $resizeResult->get_error_message() ) );
            }
        }
    }

    return $params;
}

/*
 * On media remove, also remove it from plugin's "backup" folder, if exists.
 */
function wpimages_handle_remove($post_id){
    $meta = wp_get_attachment_metadata( $post_id );
    if( isset( $meta['file'] ) ){
        $uploads = wp_upload_dir();
        $file = $uploads['basedir'] . '/optimisationio_media_backup/' . basename( $meta['file'] );
        if( file_exists( $file ) ){
            unlink( $file );
        }
    }
    return $post_id;
}

/**
 * read in the image file from the params and then save as a new jpg file.
 * if successful, remove the original image and alter the return
 * parameters to return the new jpg instead of the original
 *
 * @param string 'bmp' or 'png'
 * @param array $params
 * @return array altered params
 */
function wpimages_convert_to_jpg($type, $params) {

    $img = null;

    if ('bmp' === $type) {
        include_once 'libs/imagecreatefrombmp.php';
        $img = imagecreatefrombmp($params['file']);
    }
    elseif ('png' === $type) {
        if (!function_exists('imagecreatefrompng')) {
            return wp_handle_upload_error($params['file'], 'wpimages_convert_to_jpg requires gd library enabled');
        }
        $img = imagecreatefrompng($params['file']);
    }
    else {
        return wp_handle_upload_error($params['file'], 'Unknown image type specified in wpimages_convert_to_jpg');
    }

    // we need to change the extension from the original to .jpg so we have to ensure it will be a unique filename
    $uploads     = wp_upload_dir();
    $oldFileName = basename($params['file']);
    $newFileName = basename(str_ireplace("." . $type, ".jpg", $oldFileName));
    $newFileName = wp_unique_filename($uploads['path'], $newFileName);

    $quality = wpimages_get_option('wpimages_quality', WPIMAGE_DEFAULT_QUALITY);

    if (imagejpeg($img, $uploads['path'] . '/' . $newFileName, $quality)) {
        // conversion succeeded.  remove the original bmp & remap the params
        unlink($params['file']);

        $params['file'] = $uploads['path'] . '/' . $newFileName;
        $params['url']  = $uploads['url'] . '/' . $newFileName;
        $params['type'] = 'image/jpeg';
    } else {
        unlink($params['file']);

        return wp_handle_upload_error($oldPath, __("Oh Snap! Wp image resizer was Unable to process the $type file.  "
            . "If you continue to see this error you may need to disable the $type-To-JPG "
            . "feature in Wp image convertor settings.", 'wpimage'));
    }

    return $params;
}

function stripe_payment() {

    $email = $_POST['email'];
    $price = $_POST['price'];
    $token = $_POST['token'];
    $description = $_POST['description'];

    \WpimageCompression_Stripe\WpimageCompression_Stripe::setApiKey(STRIPE_APIKEY);
    $charge = \WpimageCompression_Stripe\Charge::create(array(
      "amount" => $price,
      "currency" => "gbp",
      "source" => $token, // obtained with Stripe.js
      "description" => $description
    ));
    if($charge->paid == true) {
        //add the credits
        global $wpdb;
        $exist = $wpdb->get_row("select * from ".$wpdb->prefix."image_compression_settings", ARRAY_A);
        if($price == 1000){
           $add = 1000;
        }else if($price == 4000) {
           $add = 5000;
        }else if($price == 8000) {
            $add = 10000;
        }else if($price == 15000){
            $add = 20000;
        }
        if ($exist) {
            $array = array();
            $array['total_allowed'] = $exist['total_allowed']+$add;
            $wpdb->update($wpdb->prefix."image_compression_settings", $array, array('id'=>$exist['id']));
        } else {
            $aray = array();
            $array['total_allowed'] = 100+$add;
            $wpdb->insert($wpdb->prefix."image_compression_settings", $array);
        }
        $return['type'] = 'success';
        $return['msg'] = 'Api updated successfully';
        echo json_encode($return);die;

    }else{
        //return error message
        $return['type'] = 'error';
        $return['msg'] = 'Sorry could not update';
        echo json_encode($return);die;

    }
}

function wpimages_replace_media_image( $image_path, $compressed_url ){
    $rv = false;
    $ch = curl_init($compressed_url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
    $result = curl_exec($ch);
    $rv     = file_put_contents($image_path, $result);
    return $rv !== false;
}

function wpimages_optimize_image( $image_path, $backup ) {
    $quality = get_option( 'wpimages_quality', WPIMAGE_DEFAULT_QUALITY );
    $quality_auto = get_option( 'wpimages_quality_auto', 'auto' );
    $wpimage  = new Wpimage();
    return $wpimage->upload( array(
        "file" => $image_path,
        "origin" => "wp",
        'backup_before_compression' => $backup,
        'wpimages_quality' => 'manual' === $quality_auto ? $quality : $quality_auto,
    ));
}

function wpimages_pretty_kb( $bytes ){
    return round( ( $bytes / 1024 ), 2 ) . ' kB';
}

/* add filters to hook into uploads */
add_filter('wp_handle_upload', 'wpimages_handle_upload');

/* add filters to hook images remove */
add_filter('delete_attachment', 'wpimages_handle_remove');

/* add filters/actions to customize upload page */

// TODO: if necessary to update the post data in the future...
// add_filter( 'wp_update_attachment_metadata', 'wpimages_handle_update_attachment_metadata' );

if ( ! class_exists( 'Wp_Image_compression' ) ) {
    require_once dirname( __FILE__ ) . '/lib/class-wp-image-compression.php';
}

new Wp_Image_compression();
